'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var messagechanneladapter = require('comlinkjs/umd/messagechanneladapter');

// handle the difference between EventEmitter(node) and EventTarget(web)
const proxyByListener = new WeakMap();
function applyEventAdapter(target, emitter) {
    Object.assign(target, {
        addEventListener: function (type, listener) {
            const proxy = function (data) {
                listener({ data });
            };
            proxyByListener.set(listener, proxy);
            (emitter || this).addListener(type, proxy);
        },
        removeEventListener: function (type, listener) {
            const proxy = proxyByListener.get(listener);
            if (!proxy) {
                return;
            }
            proxyByListener.delete(listener);
            (emitter || this).removeListener(type, proxy);
        },
    });
    return target;
}
function patchCommon() {
    global.self = global;
}

// polyfill MessagePort and MessageChannel
class MessagePortPolyfill {
    constructor() {
        this.onmessage = null;
        this.onmessageerror = null;
        this.otherPort = null;
        this.onmessageListeners = [];
    }
    dispatchEvent(event) {
        if (this.onmessage) {
            this.onmessage(event);
        }
        this.onmessageListeners.forEach(listener => listener(event));
        return true;
    }
    postMessage(message) {
        if (!this.otherPort) {
            return;
        }
        this.otherPort.dispatchEvent({ data: message });
    }
    addEventListener(type, listener) {
        if (type !== 'message') {
            return;
        }
        if (typeof listener !== 'function' ||
            this.onmessageListeners.indexOf(listener) !== -1) {
            return;
        }
        this.onmessageListeners.push(listener);
    }
    removeEventListener(type, listener) {
        if (type !== 'message') {
            return;
        }
        const index = this.onmessageListeners.indexOf(listener);
        if (index === -1) {
            return;
        }
        this.onmessageListeners.splice(index, 1);
    }
    start() {
        // do nothing at this moment
    }
    close() {
        // do nothing at this moment
    }
}
class MessageChannelPolyfill {
    constructor() {
        this.port1 = new MessagePortPolyfill();
        this.port2 = new MessagePortPolyfill();
        this.port1.otherPort = this.port2;
        this.port2.otherPort = this.port1;
    }
}

class NodeMessageAdapter {
    constructor(worker) {
        this.wrap = null;
        // use child process id as wrap id
        let wrapId = '';
        let smc = null;
        if (worker) {
            wrapId = `${worker.pid}`;
            // main process
            smc = applyEventAdapter({
                send(message) {
                    worker.send(message);
                }
            }, worker);
        }
        else {
            // child process
            wrapId = `${process.pid}`;
            smc = applyEventAdapter({
                send(message) {
                    process.send(message);
                },
            }, process);
        }
        this.wrap = messagechanneladapter.wrap(smc, wrapId);
    }
    postMessage(message, transferList) {
        this.wrap.postMessage(message);
    }
    addEventListener(type, listener) {
        this.wrap.addEventListener(type, listener);
    }
    removeEventListener(type, listener) {
        this.wrap.removeEventListener(type, listener);
    }
}
function patchMessageChannel() {
    patchCommon();
    global.MessageChannel = MessageChannelPolyfill;
    global.MessagePort = MessagePortPolyfill;
}

exports.NodeMessageAdapter = NodeMessageAdapter;
exports.patchMessageChannel = patchMessageChannel;
