'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var worker_threads = require('worker_threads');

// handle the difference between EventEmitter(node) and EventTarget(web)
const proxyByListener = new WeakMap();
function applyEventAdapter(target, emitter) {
    Object.assign(target, {
        addEventListener: function (type, listener) {
            const proxy = function (data) {
                listener({ data });
            };
            proxyByListener.set(listener, proxy);
            (emitter || this).addListener(type, proxy);
        },
        removeEventListener: function (type, listener) {
            const proxy = proxyByListener.get(listener);
            if (!proxy) {
                return;
            }
            proxyByListener.delete(listener);
            (emitter || this).removeListener(type, proxy);
        },
    });
    return target;
}
function patchCommon() {
    global.self = global;
}

// adapter for main thread
class NodeMessageAdapter {
    constructor(worker) {
        this.worker = null;
        this.eventEmitter = null;
        this.worker = worker || worker_threads.parentPort;
        this.eventEmitter = applyEventAdapter({}, this.worker);
    }
    postMessage(message, transferList) {
        this.worker.postMessage(message, transferList);
    }
    addEventListener(type, listener) {
        this.eventEmitter.addEventListener(type, listener);
    }
    removeEventListener(type, listener) {
        this.eventEmitter.removeEventListener(type, listener);
    }
}
function patchMessageChannel() {
    patchCommon();
    global.MessageChannel = worker_threads.MessageChannel;
    global.MessagePort = worker_threads.MessagePort;
    applyEventAdapter(worker_threads.MessagePort.prototype);
}

exports.NodeMessageAdapter = NodeMessageAdapter;
exports.patchMessageChannel = patchMessageChannel;
